package org.hardknockjava.lesson26.example01;

import java.awt.Desktop;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;

public class PageDownload implements Runnable {
    // address is the address of the web page we want to open.
    // exception is the Throwable thrown during execution of the
    // thread, if any.
    // sequence is the sequence in which this thread was started.
    private String address = null;
    private Throwable exception = null;
    private int sequence = 0;

    private List<PageDownload> completedDownloadList;

    public PageDownload(String address, int sequence, List<PageDownload> completedDownloadList) {
        this.address = address;
        this.sequence = sequence;
        this.completedDownloadList = completedDownloadList;
    }

    @Override
    public void run() {
        File htmlFile = null;

        try {
            System.out.println("Starting thread sequence " + sequence + " for " + address);

            File htmlDir = new File("html");
            htmlDir.mkdirs();
            htmlFile = File.createTempFile("temp_", ".html", htmlDir);
            processDownload(htmlFile);
        } catch (Throwable e) {
            exception = e;
        } finally {
            if (htmlFile.exists()) {
                try {
                    Thread.sleep(1500);
                } catch (InterruptedException e) {
                }
                try {
                    htmlFile.delete();
                } catch (SecurityException e) {
                    System.out.println(e.getMessage());
                }
            }
        }
    }

    private void processDownload(File htmlFile) {
        try {
            StringBuilder html = new StringBuilder();
            BufferedWriter wtr = new BufferedWriter(new FileWriter(htmlFile));

            // URL, and its openStream() method, are the basis for reading from
            // a file on the Internet.
            URL url = new URL(address);
            URLConnection conn = url.openConnection();
            InputStream in = conn.getInputStream();
            BufferedReader rdr = new BufferedReader(new InputStreamReader(in));

            // We're going to copy the file from the Internet to a
            // temporary file on disk, and then display it in our
            // web browser.
            char[] buffer = new char[2048];
            int length = 0;
            while ((length = rdr.read(buffer)) >= 0) {
                html.append(buffer, 0, length);
                wtr.write(buffer, 0, length);
            }
            System.out.println(html.length() + " bytes were read from " + address + ".");

            rdr.close();
            wtr.flush();
            wtr.close();

            // If a web browser is available, call the browse() method
            // to display the HTML file we've written.
            if (Desktop.isDesktopSupported() && Desktop.getDesktop().isSupported(Desktop.Action.BROWSE)) {
                Desktop.getDesktop().browse(htmlFile.toURI());
            } else {
                System.out.println("Browsing is not supported on your system.");
            }

            // Add this instance to completedDownloadList and issue
            // notifyAll.
            synchronized (completedDownloadList) {
                completedDownloadList.add(this);
                completedDownloadList.notifyAll();
            }

        } catch (Throwable e) {
            this.exception = e;
        }
    }

    public Throwable getException() {
        return this.exception;
    }

    public String getAddress() {
        return address;
    }

    public int getSequence() {
        return sequence;
    }
}
