package org.hardknockjava.lesson26.example01;

import java.util.ArrayList;
import java.util.List;

public class PageDownloadDriver {
    private static final String DECLARATION_URL = "https://www.archives.gov/founding-docs/declaration-transcript";
    private static final String YAHOO_URL = "https://www.yahoo.com";
    private static final String AOL_URL = "https://www.aol.com";

    private static final String[] ADDRESS_ARRAY = //
            { AOL_URL //
                    , YAHOO_URL //
                    , DECLARATION_URL };

    // List of active PageDownload instances.
    private List<PageDownload> liveDownloadList = new ArrayList<PageDownload>();
    
    // List of completed PageDownload instances, shared with PageDownload.
    private List<PageDownload> completedDownloadList = new ArrayList<PageDownload>();

    public static void main(String[] args) throws Exception {
        PageDownloadDriver driver = new PageDownloadDriver();
        driver.execute();
    }

    private void execute() {
        int sequence = 0;
        for (String address : ADDRESS_ARRAY) {
            PageDownload download = new PageDownload(address, ++sequence, completedDownloadList);
            liveDownloadList.add(download);
        }

        // Start threads for each instance in liveDownloadList.
        for (PageDownload download : liveDownloadList) {
            new Thread(download).start();
        }

        // Repeat this loop until there are no more instances
        // in liveDownloadList.
        while (!liveDownloadList.isEmpty()) {
            // Wait for one or more of the threads to notify.
            try {
                synchronized (completedDownloadList) {
                    if (completedDownloadList.isEmpty()) {
                        completedDownloadList.wait();
                    }
                }
            } catch (InterruptedException e) {
            }

            // Copy instances from completedDownloadList to a temporary list,
            // then clear completedDownloadList.
            List<PageDownload> tempList;
            synchronized (completedDownloadList) {
                tempList = new ArrayList<PageDownload>(completedDownloadList);
                completedDownloadList.clear();
            }

            // Report on the status of the completed downloads.
            for (PageDownload instance : tempList) {
                System.out.println("Thread for URL " + instance.getAddress() + " sequence " + instance.getSequence()
                        + " has completed.");
                if (instance.getException() != null) {
                    System.out.println("Exception occured in thread for URL: " + instance.getAddress());
                    instance.getException().printStackTrace();
                }
            }

            // Remove the completed downloads from liveDownloadList.
            // If this empties liveDownloadList, we're done.
            liveDownloadList.removeAll(tempList);
        }

        System.out.println("Done!");
    }
}
